
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/


#if UNITY_EDITOR || UNITY_IPHONE || UNITY_ANDROID
    #define UNITY_PLATFORM
#endif

#if __ANDROID__
using Android.App;
#endif
#if UNITY_PLATFORM
using UnityEngine;
#endif
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using com.amazon.whisperplay.fling.cpt.media.service.json;

namespace com.amazon.whisperplay.fling.cpt.media.service 
{
    /// <summary> Provides a lazy-initialized singleton implementation of IAmazonFlingPlayerService. </summary> 

#if UNITY_PLATFORM
    public abstract partial class AmazonFlingPlayerServiceImpl : MonoBehaviour, IAmazonFlingPlayerService
#else
    public abstract partial class AmazonFlingPlayerServiceImpl : IAmazonFlingPlayerService
#endif
    {
        protected delegate void CallbackDelegate(string jsonMessage);
        static AmazonLogger logger;
        static readonly Dictionary<string, IDelegator> callbackDictionary = new Dictionary<string, IDelegator>();
        static readonly System.Object callbackLock = new System.Object();
        static readonly Dictionary<string, List<IDelegator>> eventListeners = new Dictionary<string, List<IDelegator>>();
        static readonly System.Object eventLock = new System.Object();
        
        private AmazonFlingPlayerServiceImpl() {}
        
        public static IAmazonFlingPlayerService Instance {
            get
            { 
                return Builder.instance; 
            }
        }

        private class Builder
        {
            // A static constructor tells the C# compiler not to mark type as beforefieldinit, and thus lazy load this class
            static Builder() {}

            internal static readonly IAmazonFlingPlayerService instance =
#if UNITY_EDITOR
                new AmazonFlingPlayerServiceUnityEditor();
#elif UNITY_ANDROID
                AmazonFlingPlayerServiceUnityAndroid.Instance;
#elif __ANDROID__
                AmazonFlingPlayerServiceAndroid.Instance;
#else
                new AmazonFlingPlayerServiceDefault();
#endif
        }
#if __IOS__
        [MonoTouch.MonoPInvokeCallbackAttribute (typeof (CallbackDelegate))]
#endif
        public static void callback(string jsonMessage) 
        {
            Dictionary<string, object> message = null;
            try 
            {
                logger.Debug ("Executing callback");
                message = Json.Deserialize(jsonMessage) as Dictionary<string, object>;
                string callerId = message["callerId"] as string;
                Dictionary<string, object> response = message["response"] as Dictionary<string, object>;
                callbackCaller(response, callerId);
            }
            catch (KeyNotFoundException e)
            {
                logger.Debug("callerId not found in callback");
                throw new AmazonException("Internal Error: Unknown callback id", e);
            }
            catch (AmazonException e)
            {
                logger.Debug("Async call threw exception: " + e.ToString());
            }
        }

        private static void callbackCaller(Dictionary<string, object> response, string callerId)
        {
            IDelegator delegator = null;

            try
            {
                Jsonable.CheckForErrors(response);
                
                lock(AmazonFlingPlayerServiceImpl.callbackLock)
                {
                    delegator = AmazonFlingPlayerServiceImpl.callbackDictionary[callerId];
                    AmazonFlingPlayerServiceImpl.callbackDictionary.Remove(callerId);
                
                    delegator.ExecuteSuccess(response);
                }
            }
            catch (AmazonException e)
            {
                 lock(AmazonFlingPlayerServiceImpl.callbackLock)
                 {
                    if (delegator == null) {
                        delegator = AmazonFlingPlayerServiceImpl.callbackDictionary[callerId];                        
                    }
                    AmazonFlingPlayerServiceImpl.callbackDictionary.Remove(callerId);                 
                    
                    delegator.ExecuteError(e);
                 }
            }
        }

#if __IOS__
        [MonoTouch.MonoPInvokeCallbackAttribute (typeof (CallbackDelegate))]
#endif
        public static void FireEvent(string jsonMessage)
        {
            try
            {
                logger.Debug("eventReceived");
                Dictionary<string, object> jsonDict = Json.Deserialize(jsonMessage) as Dictionary<string, object>;
                string eventId = jsonDict["eventId"] as string;
                Dictionary<string, object> response = null;
                
                if(jsonDict.ContainsKey("response"))
                {
                    response = jsonDict["response"] as Dictionary<string, object>;
                    Jsonable.CheckForErrors(response);
                }

                lock(eventLock)
                {
                    foreach (var delegator in eventListeners[eventId])
                    {
                        if(response != null)
                        {
                            delegator.ExecuteSuccess(response);
                        }
                        else
                        {
                            delegator.ExecuteSuccess();
                        }
                    }
                }
            }
            catch (AmazonException e)
            {
                logger.Debug("Event call threw exception: " + e.ToString());
            }
        }
        // AmazonFlingPlayerService API
        public abstract void StartService(StartServiceParameters startServiceParameters);
        public abstract void UpdateVolume(Volume volume);
        public abstract void UpdateMute(Mute mute);
        public abstract void UpdatePosition(Position position);
        public abstract void UpdateDuration(Duration duration);
        public abstract void UpdateMediaState(MediaState mediaState);
        public abstract void UpdateMediaCondition(MediaCondition mediaCondition);
        public abstract void AddSupportedMimeType(MimeType mimeType);
        public abstract void UpdateMediaPlayerInfo(MediaPlayerInfo mediaPlayerInfo);
        public abstract void UnityFireEvent(string jsonMessage);
        // Event API
        public abstract void AddSetVolumeListener (SetVolumeDelegate responseDelegate);
        public abstract void RemoveSetVolumeListener (SetVolumeDelegate responseDelegate);
        public abstract void AddSetMuteListener (SetMuteDelegate responseDelegate);
        public abstract void RemoveSetMuteListener (SetMuteDelegate responseDelegate);
        public abstract void AddPauseListener (PauseDelegate responseDelegate);
        public abstract void RemovePauseListener (PauseDelegate responseDelegate);
        public abstract void AddPlayListener (PlayDelegate responseDelegate);
        public abstract void RemovePlayListener (PlayDelegate responseDelegate);
        public abstract void AddStopListener (StopDelegate responseDelegate);
        public abstract void RemoveStopListener (StopDelegate responseDelegate);
        public abstract void AddSeekListener (SeekDelegate responseDelegate);
        public abstract void RemoveSeekListener (SeekDelegate responseDelegate);
        public abstract void AddSetMediaSourceListener (SetMediaSourceDelegate responseDelegate);
        public abstract void RemoveSetMediaSourceListener (SetMediaSourceDelegate responseDelegate);
        public abstract void AddSetPlayerStyleListener (SetPlayerStyleDelegate responseDelegate);
        public abstract void RemoveSetPlayerStyleListener (SetPlayerStyleDelegate responseDelegate);
        public abstract void AddSendCommandListener (SendCommandDelegate responseDelegate);
        public abstract void RemoveSendCommandListener (SendCommandDelegate responseDelegate);

#if __ANDROID__
        public abstract void SetCurrentAndroidActivity(Activity activity);
#endif
    }
}

