﻿/* 
* Copyright 2015 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;
using com.amazon.whisperplay.fling.cpt.media.service;
using com.amazon.whisperplay.fling.cpt.media.service.json;

public class FlingPlayer : MonoBehaviour {
	public AudioSource source;
	public Image coverArtImage;
	public Text titleText;
	public Text descriptionText;
	IAmazonFlingPlayerService flingPlayerService;
	MediaState mediaState;
	Position position;

	// Use this for initialization
	void Start () {

		if (flingPlayerService == null) {
			flingPlayerService = AmazonFlingPlayerServiceImpl.Instance;
			flingPlayerService.AddSendCommandListener(SendCommandCallback);
			flingPlayerService.AddPauseListener(PauseCallback);
			flingPlayerService.AddPlayListener(PlayCallback);
			flingPlayerService.AddStopListener(StopCallback);
			flingPlayerService.AddSeekListener(SeekCallback);
			flingPlayerService.AddSetMediaSourceListener(SetMediaSourceCallback);

			StartServiceParameters startServiceParameters = new StartServiceParameters();
			startServiceParameters.PlayerId = "amzn.wp.cpt.test";
			flingPlayerService.StartService(startServiceParameters);

			position = new Position ();
			position.PositionLong = 0;
			flingPlayerService.UpdatePosition (position);

			//set mediastate
			mediaState = new MediaState ();
			mediaState.State = State.NOSOURCE;
			flingPlayerService.UpdateMediaState (mediaState);

			//set mediacondition
			MediaCondition mediaCondition = new MediaCondition();
			mediaCondition.Condition = Condition.GOOD;
			flingPlayerService.UpdateMediaCondition(mediaCondition);
		}
	}
	
	private void SendCommandCallback(Command command) {
		descriptionText.text = command.CommandString;
	}
	
	private void SeekCallback(SeekParameters seekParameters) {
		State previousState = mediaState.State;
		//update mediastate
		mediaState.State = State.SEEKING;
		flingPlayerService.UpdateMediaState (mediaState);

		if (seekParameters.PlayerSeekMode == PlayerSeekMode.PSM_ABSOLUTE) {
			source.time = (float) seekParameters.SeekValue / 1000;
		} else {
			source.time += (float) seekParameters.SeekValue / 1000;
		}
		
		//update mediastate
		mediaState.State = previousState;
		flingPlayerService.UpdateMediaState (mediaState);
	}

	private void PauseCallback() {
		source.Pause ();

		//update mediastate
		mediaState.State = State.PAUSED;
		flingPlayerService.UpdateMediaState (mediaState);
	}

	IEnumerator WaitForReadyToPlay() {
		yield return new WaitForSeconds(0.1f);
	}
	
	private void PlayCallback() {

		// Make sure the clip is ready to start before we start playing
		while (source.clip.loadState == AudioDataLoadState.Loading || source.clip.loadState == AudioDataLoadState.Unloaded) {
			StartCoroutine(WaitForReadyToPlay());
		}
		source.Play();

		//update mediastate
		mediaState.State = State.PLAYING;
		flingPlayerService.UpdateMediaState (mediaState);
	}
	
	private void StopCallback() {
		source.Stop ();

		//update mediastate
		mediaState.State = State.FINISHED;
		flingPlayerService.UpdateMediaState (mediaState);
	}

	IEnumerator LoadMetaData(string metadata) {
		var node = Json.Deserialize(metadata) as Dictionary<string, System.Object>;
		titleText.text = node["title"] as string;
		descriptionText.text = node["description"] as string;

		WWW www = new WWW(node["poster"] as string);
		yield return www;

		Sprite sprite = Sprite.Create(www.texture, new Rect(0,0, www.texture.width, www.texture.height), new Vector2(0.5f, 0.5f));
		coverArtImage.sprite = sprite;
	}
	
	private void SetMediaSourceCallback(SetMediaSourceParameters setMediaSourceParameters) {
		//update mediastate
		mediaState.State = State.PREPARINGMEDIA;
		flingPlayerService.UpdateMediaState (mediaState);

		//update mediaplayerinfo
		MediaPlayerInfo mediaPlayerInfo = new MediaPlayerInfo ();
		mediaPlayerInfo.Source = setMediaSourceParameters.Source;
		mediaPlayerInfo.Metadata = setMediaSourceParameters.MetadataJson;
		flingPlayerService.UpdateMediaPlayerInfo(mediaPlayerInfo);

		Debug.Log ("FlingPlayerScript, starting loadmetadata coroutine");
		// load metadata
		StartCoroutine(LoadMetaData (setMediaSourceParameters.MetadataJson));

		// Start download
		WWW www = new WWW(setMediaSourceParameters.Source);

		// Assign clip to audio source and Sync playback with audio
		source.clip = www.GetAudioClip(false, true);
		Debug.Log ("FlingPlayerScript, state: " + source.clip.loadState.ToString());

		// Make sure the clip is ready to start
		while (source.clip.loadState == AudioDataLoadState.Loading || source.clip.loadState == AudioDataLoadState.Unloaded) {
			Debug.Log ("FlingPlayerScript, load state: " + source.clip.loadState.ToString());
			StartCoroutine(WaitForReadyToPlay());
		}
		//update duration;
		Duration duration = new Duration ();
		duration.DurationLong = (long) source.clip.length * 1000;
		Debug.Log ("FlingPlayerScript, duration set to: " + duration.DurationLong.ToString());
		flingPlayerService.UpdateDuration(duration);

		//update mediastate
		mediaState.State = State.READYTOPLAY;
		flingPlayerService.UpdateMediaState (mediaState);

		if (setMediaSourceParameters.AutoPlay == true) {
			PlayCallback();
		}
	}

	void Update() {
		if (source != null && source.isPlaying) {
			if ((long)(source.time * 1000) != position.PositionLong) {
				//update position;
				position.PositionLong = (long)source.time * 1000;
				flingPlayerService.UpdatePosition (position);
			}
		} else if (mediaState.State == State.PLAYING) {
			//playback has finished
			//update mediastate
			mediaState.State = State.FINISHED;
			flingPlayerService.UpdateMediaState (mediaState);

			//reset clip position
			if (source != null) {
				source.time = 0;
			}
		}
	}
}
