
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/


#if __ANDROID__
using Android.App;
#endif
using System.Collections.Generic;
using System.Diagnostics;
using com.amazon.whisperplay.fling.cpt.media.controller.json;

namespace com.amazon.whisperplay.fling.cpt.media.controller 
{
    public abstract partial class AmazonFlingServiceImpl
    {
        private abstract class AmazonFlingServiceBase : AmazonFlingServiceImpl
        {
            readonly System.Random random = new System.Random();
            static readonly System.Object startLock = new System.Object();
            static volatile bool startCalled = false;
        
            protected void Start () 
            {
                if(startCalled) 
                {
                    return;
                }
            
                lock(startLock)
                {
                    if(startCalled == false)
                    {
                        Init();
                        RegisterCallback();
                        RegisterEventListener();
                        RegisterCrossPlatformTool();
                        startCalled = true;
                    }
                }
            }

            protected abstract void Init();
            protected abstract void RegisterCallback();
            protected abstract void RegisterEventListener();
            protected abstract void RegisterCrossPlatformTool();
            
            public AmazonFlingServiceBase()
            {
                logger = new AmazonLogger(this.GetType().Name);
            }

            public override void UnityCallback(string jsonMessage)
            {
                AmazonFlingServiceImpl.callback(jsonMessage);
            }

            public override void UnityFireEvent(string jsonMessage)
            {
                AmazonFlingServiceImpl.FireEvent(jsonMessage);
            }

            private string uuid()
            {
               //a versionless uuid generator
               var id = string.Format ("{0:X4}{1:X4}-{2:X4}-{3:X4}-{4:X4}-{5:X4}{6:X4}{7:X4}",
                                           (byte)random.Next (0, 255),
                                           (byte)random.Next (0, 255),
                                           (byte)random.Next (0, 255),
                                           (byte)random.Next (0, 255),
                                           (byte)random.Next (0, 255),
                                           (byte)random.Next (0, 255),
                                           (byte)random.Next (0, 255),
                                           (byte)random.Next (0, 255));
               return id;
            }

            public override void StartDiscovery(PlayerServiceId playerServiceId)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(StartDiscoveryJson(playerServiceId.ToJson())) as Dictionary<string, object>);
            }

            private string StartDiscoveryJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeStartDiscoveryJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeStartDiscoveryJson(string jsonMessage);

            public override void StopDiscovery()
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(StopDiscoveryJson("{}")) as Dictionary<string, object>);
            }

            private string StopDiscoveryJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeStopDiscoveryJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeStopDiscoveryJson(string jsonMessage);

            public override void GetVolume(Player player, DoubleResultDelegate doubleResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new DoubleResultDelegator(doubleResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeGetVolumeJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeGetVolumeJson(string jsonMessage, string callerId);
            public override void SetVolume(SetVolumeParameters setVolumeParameters, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeSetVolumeJson(setVolumeParameters.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeSetVolumeJson(string jsonMessage, string callerId);
            public override void IsMute(Player player, BooleanResultDelegate booleanResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new BooleanResultDelegator(booleanResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeIsMuteJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeIsMuteJson(string jsonMessage, string callerId);
            public override void SetMute(MuteParameters muteParameters, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeSetMuteJson(muteParameters.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeSetMuteJson(string jsonMessage, string callerId);
            public override void GetPosition(Player player, LongResultDelegate longResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new LongResultDelegator(longResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeGetPositionJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeGetPositionJson(string jsonMessage, string callerId);
            public override void GetDuration(Player player, LongResultDelegate longResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new LongResultDelegator(longResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeGetDurationJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeGetDurationJson(string jsonMessage, string callerId);
            public override void GetStatus(Player player, StatusDelegate statusDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new StatusDelegator(statusDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeGetStatusJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeGetStatusJson(string jsonMessage, string callerId);
            public override void IsMimeTypeSupported(IsMimeTypeSupportedParameters isMimeTypeSupportedParameters, BooleanResultDelegate booleanResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new BooleanResultDelegator(booleanResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeIsMimeTypeSupportedJson(isMimeTypeSupportedParameters.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeIsMimeTypeSupportedJson(string jsonMessage, string callerId);
            public override void Pause(Player player, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativePauseJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativePauseJson(string jsonMessage, string callerId);
            public override void Play(Player player, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativePlayJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativePlayJson(string jsonMessage, string callerId);
            public override void Stop(Player player, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeStopJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeStopJson(string jsonMessage, string callerId);
            public override void Seek(PlayerSeekParameters playerSeekParameters, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeSeekJson(playerSeekParameters.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeSeekJson(string jsonMessage, string callerId);
            public override void SetMediaSource(SetMediaSourceParameters setMediaSourceParameters, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeSetMediaSourceJson(setMediaSourceParameters.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeSetMediaSourceJson(string jsonMessage, string callerId);
            public override void SetPlayerStyle(SetPlayerStyleParameters setPlayerStyleParameters, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeSetPlayerStyleJson(setPlayerStyleParameters.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeSetPlayerStyleJson(string jsonMessage, string callerId);
            public override void AddStatusListener(Player player, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeAddStatusListenerJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeAddStatusListenerJson(string jsonMessage, string callerId);
            public override void RemoveStatusListener(Player player, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeRemoveStatusListenerJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeRemoveStatusListenerJson(string jsonMessage, string callerId);
            public override void SetPositionUpdateInterval(SetPositionUpdateIntervalParameters setPositionUpdateIntervalParameters, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeSetPositionUpdateIntervalJson(setPositionUpdateIntervalParameters.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeSetPositionUpdateIntervalJson(string jsonMessage, string callerId);
            public override void SendCommand(SendCommandParameters sendCommandParameters, VoidResultDelegate voidResultDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new VoidResultDelegator(voidResultDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeSendCommandJson(sendCommandParameters.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeSendCommandJson(string jsonMessage, string callerId);
            public override void GetMediaInfo(Player player, PlayerInfoDelegate playerInfoDelegate = null, AmazonExceptionDelegate amazonExceptionDelegate = null)
            {
                Start();
                string callerId = uuid();
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    AmazonFlingServiceImpl.callbackDictionary.Add(callerId, new PlayerInfoDelegator(playerInfoDelegate, amazonExceptionDelegate));
                }
                logger.Debug ("Calling (async) native code");
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeGetMediaInfoJson(player.ToJson(), callerId);

                stopwatch.Stop();
                Jsonable.CheckForErrors(Json.Deserialize(result) as Dictionary<string, object>);
                logger.Debug (string.Format("Successfully called (async) native code in {0} ms", stopwatch.ElapsedMilliseconds));
            }

            protected abstract string NativeGetMediaInfoJson(string jsonMessage, string callerId);

            public override void AddPlayerDiscoveredListener(PlayerDiscoveredDelegate responseDelegate)
            {
                Start();
                string eventId = "playerDiscovered";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new PlayerDiscoveredDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new PlayerDiscoveredDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemovePlayerDiscoveredListener(PlayerDiscoveredDelegate responseDelegate)
            {
                Start();
                string eventId = "playerDiscovered";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(PlayerDiscoveredDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddPlayerLostListener(PlayerLostDelegate responseDelegate)
            {
                Start();
                string eventId = "playerLost";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new PlayerLostDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new PlayerLostDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemovePlayerLostListener(PlayerLostDelegate responseDelegate)
            {
                Start();
                string eventId = "playerLost";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(PlayerLostDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddDiscoveryFailureListener(DiscoveryFailureDelegate responseDelegate)
            {
                Start();
                string eventId = "discoveryFailure";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new DiscoveryFailureDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new DiscoveryFailureDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveDiscoveryFailureListener(DiscoveryFailureDelegate responseDelegate)
            {
                Start();
                string eventId = "discoveryFailure";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(DiscoveryFailureDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddOnStatusChangeListener(OnStatusChangeDelegate responseDelegate)
            {
                Start();
                string eventId = "onStatusChange";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new OnStatusChangeDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new OnStatusChangeDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveOnStatusChangeListener(OnStatusChangeDelegate responseDelegate)
            {
                Start();
                string eventId = "onStatusChange";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(OnStatusChangeDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }

#if __ANDROID__
            public override void SetCurrentAndroidActivity(Activity activity)
            {
                // do nothing
            }
#endif
        }
    }
}

