
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/


#if UNITY_EDITOR || UNITY_IPHONE || UNITY_ANDROID
    #define UNITY_PLATFORM
#endif

#if __ANDROID__
using Android.App;
#endif
#if UNITY_PLATFORM
using UnityEngine;
#endif
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using com.amazon.whisperplay.fling.cpt.media.controller.json;

namespace com.amazon.whisperplay.fling.cpt.media.controller 
{
    /// <summary> Provides a lazy-initialized singleton implementation of IAmazonFlingService. </summary> 

#if UNITY_PLATFORM
    public abstract partial class AmazonFlingServiceImpl : MonoBehaviour, IAmazonFlingService
#else
    public abstract partial class AmazonFlingServiceImpl : IAmazonFlingService
#endif
    {
        protected delegate void CallbackDelegate(string jsonMessage);
        static AmazonLogger logger;
        static readonly Dictionary<string, IDelegator> callbackDictionary = new Dictionary<string, IDelegator>();
        static readonly System.Object callbackLock = new System.Object();
        static readonly Dictionary<string, List<IDelegator>> eventListeners = new Dictionary<string, List<IDelegator>>();
        static readonly System.Object eventLock = new System.Object();
        
        private AmazonFlingServiceImpl() {}
        
        public static IAmazonFlingService Instance {
            get
            { 
                return Builder.instance; 
            }
        }

        private class Builder
        {
            // A static constructor tells the C# compiler not to mark type as beforefieldinit, and thus lazy load this class
            static Builder() {}

            internal static readonly IAmazonFlingService instance =
#if UNITY_EDITOR
                new AmazonFlingServiceUnityEditor();
#elif UNITY_ANDROID
                AmazonFlingServiceUnityAndroid.Instance;
#elif UNITY_IPHONE
                AmazonFlingServiceUnityIPhone.Instance;
#elif __ANDROID__
                AmazonFlingServiceAndroid.Instance;
#elif __IOS__
                new AmazonFlingServiceIOS();
#else
                new AmazonFlingServiceDefault();
#endif
        }
#if __IOS__
        [MonoTouch.MonoPInvokeCallbackAttribute (typeof (CallbackDelegate))]
#endif
        public static void callback(string jsonMessage) 
        {
            Dictionary<string, object> message = null;
            try 
            {
                logger.Debug ("Executing callback");
                message = Json.Deserialize(jsonMessage) as Dictionary<string, object>;
                string callerId = message["callerId"] as string;
                Dictionary<string, object> response = message["response"] as Dictionary<string, object>;
                callbackCaller(response, callerId);
            }
            catch (KeyNotFoundException e)
            {
                logger.Debug("callerId not found in callback");
                throw new AmazonException("Internal Error: Unknown callback id", e);
            }
            catch (AmazonException e)
            {
                logger.Debug("Async call threw exception: " + e.ToString());
            }
        }

        private static void callbackCaller(Dictionary<string, object> response, string callerId)
        {
            IDelegator delegator = null;

            try
            {
                Jsonable.CheckForErrors(response);
                
                lock(AmazonFlingServiceImpl.callbackLock)
                {
                    delegator = AmazonFlingServiceImpl.callbackDictionary[callerId];
                    AmazonFlingServiceImpl.callbackDictionary.Remove(callerId);
                
                    delegator.ExecuteSuccess(response);
                }
            }
            catch (AmazonException e)
            {
                 lock(AmazonFlingServiceImpl.callbackLock)
                 {
                    if (delegator == null) {
                        delegator = AmazonFlingServiceImpl.callbackDictionary[callerId];                        
                    }
                    AmazonFlingServiceImpl.callbackDictionary.Remove(callerId);                 
                    
                    delegator.ExecuteError(e);
                 }
            }
        }

#if __IOS__
        [MonoTouch.MonoPInvokeCallbackAttribute (typeof (CallbackDelegate))]
#endif
        public static void FireEvent(string jsonMessage)
        {
            try
            {
                logger.Debug("eventReceived");
                Dictionary<string, object> jsonDict = Json.Deserialize(jsonMessage) as Dictionary<string, object>;
                string eventId = jsonDict["eventId"] as string;
                Dictionary<string, object> response = null;
                
                if(jsonDict.ContainsKey("response"))
                {
                    response = jsonDict["response"] as Dictionary<string, object>;
                    Jsonable.CheckForErrors(response);
                }

                lock(eventLock)
                {
                    foreach (var delegator in eventListeners[eventId])
                    {
                        if(response != null)
                        {
                            delegator.ExecuteSuccess(response);
                        }
                        else
                        {
                            delegator.ExecuteSuccess();
                        }
                    }
                }
            }
            catch (AmazonException e)
            {
                logger.Debug("Event call threw exception: " + e.ToString());
            }
        }
        // AmazonFlingService API
        public abstract void StartDiscovery(PlayerServiceId playerServiceId);
        public abstract void StopDiscovery();
        public abstract void GetVolume(Player player, DoubleResultDelegate doubleResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void SetVolume(SetVolumeParameters setVolumeParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void IsMute(Player player, BooleanResultDelegate booleanResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void SetMute(MuteParameters muteParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void GetPosition(Player player, LongResultDelegate longResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void GetDuration(Player player, LongResultDelegate longResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void GetStatus(Player player, StatusDelegate statusDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void IsMimeTypeSupported(IsMimeTypeSupportedParameters isMimeTypeSupportedParameters, BooleanResultDelegate booleanResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void Pause(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void Play(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void Stop(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void Seek(PlayerSeekParameters playerSeekParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void SetMediaSource(SetMediaSourceParameters setMediaSourceParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void SetPlayerStyle(SetPlayerStyleParameters setPlayerStyleParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void AddStatusListener(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void RemoveStatusListener(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void SetPositionUpdateInterval(SetPositionUpdateIntervalParameters setPositionUpdateIntervalParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void SendCommand(SendCommandParameters sendCommandParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        
        public abstract void GetMediaInfo(Player player, PlayerInfoDelegate playerInfoDelegate, AmazonExceptionDelegate errorDelegate);
        
        // Method to communicate events to the application. This method is usually called by the native side, via the amazon bridge.
        public abstract void UnityCallback(string jsonMessage);
        public abstract void UnityFireEvent(string jsonMessage);
        // Event API
        public abstract void AddPlayerDiscoveredListener (PlayerDiscoveredDelegate responseDelegate);
        public abstract void RemovePlayerDiscoveredListener (PlayerDiscoveredDelegate responseDelegate);
        public abstract void AddPlayerLostListener (PlayerLostDelegate responseDelegate);
        public abstract void RemovePlayerLostListener (PlayerLostDelegate responseDelegate);
        public abstract void AddDiscoveryFailureListener (DiscoveryFailureDelegate responseDelegate);
        public abstract void RemoveDiscoveryFailureListener (DiscoveryFailureDelegate responseDelegate);
        public abstract void AddOnStatusChangeListener (OnStatusChangeDelegate responseDelegate);
        public abstract void RemoveOnStatusChangeListener (OnStatusChangeDelegate responseDelegate);

#if __ANDROID__
        public abstract void SetCurrentAndroidActivity(Activity activity);
#endif
    }
}

