
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONAmazonFlingServiceImpl.m
//

#import "AMAZONAmazonFlingServiceImpl.h"
#import "AMAZONOnStatusChangeParameters.h"
#import "AMAZONAmazonFlingServiceObjectiveCController.h"
#import "AMAZONAmazonFlingServiceStatusMonitor.h"

@implementation AMAZONAmazonFlingServiceImpl

@synthesize controller = _controller;
@synthesize deviceList = _deviceList;
@synthesize listenerList = _listenerList;

- (void)startDiscovery:(AMAZONPlayerServiceId *)playerServiceId error:(NSError *__autoreleasing *)errorPtr {
    NSLog(@"Starting discovery");
    self.deviceList = [[NSMutableDictionary alloc] init];
    self.listenerList = [[NSMutableDictionary alloc] init];
    self.discoveryController = [DiscoveryController alloc];
    [self.discoveryController searchPlayerWithId: playerServiceId.id andListener: self];
    [self.discoveryController resume];
}

- (void)deviceDiscovered:(id<RemoteMediaPlayer>)device {
    // Add device to the map based on UUID
    [self.deviceList setValue:device forKey:[device uniqueIdentifier]];
    
    // Send event to the listener
    NSLog(@"discovered %@", [device uniqueIdentifier]);
    AMAZONPlayer* player = [AMAZONPlayer alloc];
    player.playerId = [device uniqueIdentifier];
    player.playerName = [device name];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.controller firePlayerDiscovered:player];
    });
}

- (void)deviceLost:(id<RemoteMediaPlayer>)device {
    // Remove from the map based on UUID
    [self.deviceList removeObjectForKey: [device uniqueIdentifier]];
    
    // Send event to the listener
    AMAZONPlayer* player = [AMAZONPlayer alloc];
    player.playerId = [device uniqueIdentifier];
    player.playerName = [device name];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.controller firePlayerLost:player];
    });
}

- (void)discoveryFailure {
    // Send event to the listener
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.controller fireDiscoveryFailure];
    });
}

- (void)stopDiscoveryWithError:(NSError *__autoreleasing *)errorPtr {
    [self.discoveryController close];
    self.discoveryController = nil;
}

- (void)createError : (NSError *)errorPtr : (NSString*) message {
    NSString* domain = @"com.amazon.AmazonFlingService.ErrorDomain";
    NSUInteger errorCode = getVolumeError;
    NSMutableDictionary* userInfo = [NSMutableDictionary dictionary];
    [userInfo setObject:message forKey:NSLocalizedDescriptionKey];
    errorPtr = [[NSError alloc] initWithDomain:domain code:errorCode userInfo:userInfo];
}

- (void)getVolume:(AMAZONPlayer *)player completion:(futureDoubleResultBlock)completion {
    NSError* error = nil;
    AMAZONDoubleResult * ret = [[AMAZONDoubleResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"This player is unavailable"];
            completion(ret, error);
            return;
        }
        
        // Execute BFTask asynchronously
        [[device getVolume] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                ret.doubleresult = [task.result doubleValue];
                if (ret.doubleresult < 0.999) {
                    ret.doubleresult += 0.001;
                }
                return nil;
            }
            return nil;
        }];
    }
}

- (void)setVolume:(AMAZONSetVolumeParameters *)setVolumeParameters completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:setVolumeParameters.player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            completion(ret, error);
            return;
        }
        
        // Execute BFTask asynchronously
        [[device setVolume : setVolumeParameters.volume] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)isMute:(AMAZONPlayer *)player completion:(futureBooleanResultBlock)completion {
    NSError* error = nil;
    AMAZONBooleanResult * ret = [[AMAZONBooleanResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"An error occured in isMute"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"This player is unavailable"];
            completion(ret, error);
            return;
        }
        
        // Execute BFTask asynchronously
        [[device isMute] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                ret.booleanresult = [task.result boolValue];
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)setMute:(AMAZONMuteParameters *)muteParameters completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:muteParameters.player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            completion(ret, error);
            return;
        }
        
        // Execute BFTask asynchronously
        [[device setMute : muteParameters.mute] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)getPosition:(AMAZONPlayer *)player completion:(futureLongResultBlock)completion {
    NSError* error = nil;
    AMAZONLongResult * ret = [[AMAZONLongResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"An error occured in getPosition"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"This player is unavailable"];
            completion(ret, error);
            return;
        }
        
        // Execute BFTask asynchronously
        [[device getPosition] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                ret.longresult = [task.result longValue];
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)getDuration:(AMAZONPlayer *)player completion:(futureLongResultBlock)completion {
    NSError* error = nil;
    AMAZONLongResult * ret = [[AMAZONLongResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"An error occured in getDuration"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"This player is unavailable"];
            completion(ret, error);
            return;
        }
        
        // Execute BFTask asynchronously
        [[device getDuration] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                NSLog(@"Error while getting duration: %@", task.error);
                completion(ret, task.error);
            } else {
                ret.longresult = [task.result longValue];
                NSLog(@"Duration = %ld", ret.longresult);
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)getStatus:(AMAZONPlayer *)player completion:(futureStatusBlock)completion {
    NSError* error = nil;
    AMAZONStatus * ret = [[AMAZONStatus alloc] init];
    if(ret == nil) {
        [self createError : error : @"An error occured in getStatus"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"This player is unavailable"];
            completion(ret, error);
            return;
        }
        
        // Execute BFTask asynchronously
        [[device getStatus] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                MediaPlayerStatus* result = task.result;
                ret.state = [AmazonFlingServiceStatusMonitor convertState : [result state]];
                ret.condition = [AmazonFlingServiceStatusMonitor convertCondition : [result condition]];
                ret.mute = [result isMute];
                ret.muteSet = [result isMuteSet];
                ret.volume = [result volume];
                ret.volumeSet = [task.result isVolumeSet];
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)isMimeTypeSupported:(AMAZONIsMimeTypeSupportedParameters *)isMimeTypeSupportedParameters completion:(futureBooleanResultBlock)completion {
    NSError* error = nil;
    AMAZONBooleanResult * ret = [[AMAZONBooleanResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"An error occured in isMimeTypeSupported"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:isMimeTypeSupportedParameters.player.playerId];
        
        if (device == nil) {
            [self createError : error : @"This player is unavailable"];
            completion(ret, error);
        }
        
        // Execute BFTask asynchronously
        [[device isMimeTypeSupported : isMimeTypeSupportedParameters.mimeType] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                ret.booleanresult = [task.result boolValue];
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)pause:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            completion(ret, error);
        }
        
        // Execute BFTask asynchronously
        [[device pause] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)play:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            completion(ret, error);
        }
        
        // Execute BFTask asynchronously
        [[device play] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)stop:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            completion(ret, error);
        }
        
        // Execute BFTask asynchronously
        [[device stop] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)seek:(AMAZONPlayerSeekParameters *)playerSeekParameters completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:playerSeekParameters.player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            completion(ret, error);
        }
        
        SeekType mode = ABSOLUTE;
        if (playerSeekParameters.playerSeekMode == PSM_RELATIVE) {
            mode = RELATIVE;
        }
        
        // Execute BFTask asynchronously
        [[device seekToPosition: playerSeekParameters.position andMode : mode] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)setMediaSource:(AMAZONSetMediaSourceParameters *)setMediaSourceParameters completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:setMediaSourceParameters.player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            completion(ret, error);
        }
        
        // Execute BFTask asynchronously
        [[device setMediaSourceToURL: setMediaSourceParameters.source
                        metaData: setMediaSourceParameters.metadataJson
                        autoPlay: setMediaSourceParameters.autoPlay
                        andPlayInBackground: setMediaSourceParameters.playInBg] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)setPlayerStyle:(AMAZONSetPlayerStyleParameters *)setPlayerStyleParameters completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:setPlayerStyleParameters.player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            return;
        }
        
        // Execute BFTask asynchronously
        [[device setPlayerStyle: setPlayerStyleParameters.playerStyle] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)addStatusListener:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            return;
        }
        
        AmazonFlingServiceStatusMonitor* monitor = [AmazonFlingServiceStatusMonitor alloc];
        monitor.player = player;
        monitor.controller = self.controller;
        [self.listenerList setValue: monitor forKey:player.playerId];
        
        // Execute BFTask asynchronously
        [[device addStatusListener:monitor] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
                [self.listenerList removeObjectForKey:player.playerId];
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)removeStatusListener:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        AmazonFlingServiceStatusMonitor* monitor = [self.listenerList objectForKey:player.playerId];
        
        if (device == nil || monitor == nil) {
            [self createError : error : @"The player is unavailable"];
            return;
        }
        
        // Execute BFTask asynchronously
        [[device removeStatusListener:monitor] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
        
        [self.listenerList removeObjectForKey:player.playerId];
    }
}

- (void)setPositionUpdateInterval:(AMAZONSetPositionUpdateIntervalParameters *)setPositionUpdateIntervalParameters completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:setPositionUpdateIntervalParameters.player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            return;
        }
        
        // Execute BFTask asynchronously
        [[device setPositionUpdateInterval: setPositionUpdateIntervalParameters.interval] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)sendCommand:(AMAZONSendCommandParameters *)sendCommandParameters completion:(futureVoidResultBlock)completion {
    NSError* error = nil;
    AMAZONVoidResult * ret = [[AMAZONVoidResult alloc] init];
    if(ret == nil) {
        [self createError : error : @"Could not create result object"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:sendCommandParameters.player.playerId];
        
        if (device == nil) {
            [self createError : error : @"The player is unavailable"];
            return;
        }
        
        // Execute BFTask asynchronously
        [[device sendCommand: sendCommandParameters.command] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                completion(ret, error);
            }
            return nil;
        }];
    }
}

- (void)getMediaInfo:(AMAZONPlayer *)player completion:(futurePlayerInfoBlock)completion {
    NSError* error = nil;
    AMAZONPlayerInfo * ret = [[AMAZONPlayerInfo alloc] init];
    if(ret == nil) {
        [self createError : error : @"An error occured in getMediaInfo"];
        completion(ret, error);
    } else {
        // Fetch the device from the map
        id<RemoteMediaPlayer> device = [self.deviceList objectForKey:player.playerId];
        
        if (device == nil) {
            [self createError : error : @"This player is unavailable"];
            completion(ret, error);
            return;
        }
        
        // Execute BFTask asynchronously
        [[device getMediaInfo] continueWithBlock:^id(BFTask *task) {
            if (task.error != nil) {
                completion(ret, task.error);
            } else {
                MediaPlayerInfo* result = task.result;
                ret.source = [result source];
                ret.metadataJson = [result metadata];
                ret.extra = [result extra];
                completion(ret, error);
            }
            return nil;
        }];
    }
}

@end

