﻿/* 
* Copyright 2015 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/
using UnityEngine;
using System.Collections.Generic;
using com.amazon.whisperplay.fling.cpt.media.controller;
using com.amazon.whisperplay.fling.cpt.media.controller.json;
using UnityEngine.UI;
using System.Collections;
using System.IO;

public class Fling : MonoBehaviour {
	IAmazonFlingService flingService;
	SortedList<string, Player> deviceList;
	bool showDeviceMenu = false;
	Player selectedPlayer;
	Player previousPlayer;
	bool isPaused = false;
	public GameObject pickerListView;
	public GameObject pickerButton;
	public GameObject listView;
	public GameObject mediaButton;
	Sprite deviceSelectedSprite;
	Sprite noDeviceSelectedSprite;
	SetMediaSourceParameters sourceParameters;
	
	// Event Handlers
	private void PlayerDiscovered(Player args)
	{
		deviceList.Add (args.PlayerId, args);
		if (previousPlayer != null && args.PlayerId == previousPlayer.PlayerId) {
			SetDevice (args);
		}

		if (showDeviceMenu) {
			CreatePickerList();
		}
	}

	private void PlayerLost(Player args)
	{
		deviceList.Remove (args.PlayerId);
		if (showDeviceMenu) {
			CreatePickerList();
		}
	}

	private void DiscoveryFailure() {
		Debug.Log ("Discovery Failure");
	}

	private void FlingCallbackError(AmazonException e) {
		Debug.Log (e);
	}

	private void GetDurationCallback(LongResult longResult) {
		Slider positionSlider = GameObject.Find ("PositionSlider").GetComponent<Slider> ();
		positionSlider.maxValue = longResult.Longresult;

		Text durationStatus = GameObject.Find ("DurationText").GetComponent<Text> ();
		durationStatus.text = convertTime(longResult.Longresult);
	}

	private void OnStatusChange(OnStatusChangeParameters onStatusChangeParameters) {
		flingService.GetDuration(selectedPlayer, GetDurationCallback, FlingCallbackError);

		Slider positionSlider = GameObject.Find ("PositionSlider").GetComponent<Slider> ();
		positionSlider.value = onStatusChangeParameters.Position;
		
		Text mediaStatus = GameObject.Find ("MediaStatusText").GetComponent<Text> ();
		mediaStatus.text = onStatusChangeParameters.Status.State.ToString ();

		if (onStatusChangeParameters.Status.State.Equals(State.PAUSED)) {
			isPaused = true;
		} else {
			isPaused = false;
		}
		
		Text positionStatus = GameObject.Find ("PositionText").GetComponent<Text> ();
		positionStatus.text = convertTime(onStatusChangeParameters.Position);
	}
	// End Event Handlers

	//convert duration and position longs to string for display
	private string convertTime(long time) {
		long totalSecs = time / 1000;
		long hours = totalSecs / 3600;
		long minutes = (totalSecs / 60) % 60;
		long seconds = totalSecs % 60;
		string hourString = (hours == 0) ? "00" : ((hours < 10) ? "0" + hours : "" + hours);
		string minString = (minutes == 0) ? "00" : ((minutes < 10) ? "0" + minutes : "" + minutes);
		string secString = (seconds == 0) ? "00" : ((seconds < 10) ? "0" + seconds : "" + seconds);
		
		return hourString + ":" + minString + ":" + secString;
	}

	//called when canvas object is initialized
	void Start () {
		deviceList = new SortedList<string, Player>();
		if (flingService == null) {
			flingService = AmazonFlingServiceImpl.Instance;
			OnResume ();
		}
		// Register event handlers
		flingService.AddPlayerDiscoveredListener(PlayerDiscovered);
		flingService.AddPlayerLostListener(PlayerLost);
		flingService.AddDiscoveryFailureListener (DiscoveryFailure);
		flingService.AddOnStatusChangeListener(OnStatusChange);

		StartCoroutine(CreateMediaList ());

		noDeviceSelectedSprite = Resources.Load<Sprite>("ic_whisperplay_default_light_48dp");
		deviceSelectedSprite = Resources.Load<Sprite>("ic_whisperplay_default_blue_light_24dp");
	}
	
	//asynchronously creates the media list for the user to select from
	IEnumerator CreateMediaList() {
		//use the json file on the Android sdcard or the iOS app data path if it exists
		//if not, then use the json file included in the Unity resources
		string externalJSONPath = "";
		if (Application.platform == RuntimePlatform.Android) {
			Debug.Log ("Testing: setting path to sdcard");
			externalJSONPath = "/mnt/sdcard/FlingSample.json";
		} else if (Application.platform == RuntimePlatform.IPhonePlayer) {
			externalJSONPath = Application.persistentDataPath + "/FlingSample.json";
			Debug.Log ("Testing: using iOS file " + externalJSONPath);
		}
		FileInfo f = new FileInfo (externalJSONPath);
		string fileText = "";
		if (f.Exists) {
			StreamReader streamReader = File.OpenText(externalJSONPath);
			fileText = streamReader.ReadToEnd();
			streamReader.Close ();
			Debug.Log ("Testing: read text from external file " + fileText);
		} else {
			TextAsset fileAsset = (TextAsset)Resources.Load ("SampleMedia");
			fileText = fileAsset.text;
			Debug.Log ("Testing: read text from resource " + fileText);
		}

		var mediaList = Json.Deserialize(fileText) as List<System.Object>;
		((RectTransform)listView.transform).sizeDelta = new Vector2(0,300 * mediaList.Count);

		float yPosition = -150;
		foreach (Dictionary<string, System.Object> mediaNode in mediaList) {
			GameObject mediaButtonInstance = Instantiate (mediaButton) as GameObject;
			mediaButtonInstance.transform.SetParent (listView.transform);
			mediaButtonInstance.transform.localScale = new Vector3 (1, 1, 1);
			((RectTransform)mediaButtonInstance.transform).anchoredPosition = new Vector2(0,yPosition);
			((RectTransform)mediaButtonInstance.transform).sizeDelta = new Vector2(0,300);
			yPosition = yPosition - 300;
			
			AddListener(mediaButtonInstance.GetComponent<Button>(), mediaNode);
			
			GameObject titleText = mediaButtonInstance.transform.Find ("TitleText").gameObject;
			Text textComponent = titleText.GetComponent<Text>();
			textComponent.text = mediaNode["title"] as string;
			
			GameObject descriptionText = mediaButtonInstance.transform.Find ("DescriptionText").gameObject;
			textComponent = descriptionText.GetComponent<Text>();
			var metaDataNode = mediaNode["metadata"] as Dictionary<string, System.Object>;
			textComponent.text = metaDataNode["description"] as string;
			
			WWW www = new WWW(mediaNode["iconUrl"] as string);
			yield return www;
			if (www.texture.height == 8 && www.texture.width == 8) {
				Debug.Log (www.error);
				GameObject covorArtImage = mediaButtonInstance.transform.Find ("CoverArtImage").gameObject;
				Sprite sprite = Resources.Load<Sprite>("ic_whisperplay_default_light_48dp");
				covorArtImage.GetComponent<Image>().sprite = sprite;
			} else {
				GameObject covorArtImage = mediaButtonInstance.transform.Find ("CoverArtImage").gameObject;
				Sprite originalSprite = covorArtImage.GetComponent<Image>().sprite;
				Sprite sprite = Sprite.Create(www.texture, new Rect(0,0, www.texture.width, www.texture.height), originalSprite.pivot);
				covorArtImage.GetComponent<Image>().sprite = sprite;
			}
		}
	}

	void AddListener(Button button, Dictionary<string, System.Object> node) {
		button.onClick.AddListener (() => MediaSelected (node));
	}

	//called when the application comes to front or is minimized
	void OnApplicationPause (bool pause) {
		Debug.Log ("OnApplicationPause: " + pause);
		if (pause) {
			OnPause ();
		} else {
			OnResume ();
		}
	}

	//stops discovery when the application is minimized
	void OnPause (){
		if (selectedPlayer != null) {
			flingService.RemoveStatusListener(selectedPlayer, ActionlessCallback, FlingCallbackError);
		}

		flingService.StopDiscovery ();
		previousPlayer = selectedPlayer;
		selectedPlayer = null;
		deviceList.Clear ();
		if (showDeviceMenu) {
			ShowDevices(0);
		}
		Debug.Log ("Clearing Device List");
	}

	//starts discovery when the application comes to front
	void OnResume () {
		if (flingService != null) {
			PlayerServiceId serviceId = new PlayerServiceId();
			serviceId.Id = "amzn.thin.pl";
			flingService.StartDiscovery (serviceId);
		}
	}

	//Called when the user clicks the device picker button
	//Toggles the device picker screen shown by OnGUI
    public void ShowDevices (int i) {
		Debug.Log ("Device List Icon Pressed");
		showDeviceMenu = !showDeviceMenu;
		if (showDeviceMenu) {
			CreatePickerList();
		}
	}

	//creates the device picker list
	public void CreatePickerList() {
		
		Debug.Log ("Creating Picker List");
		GameObject pickerPanel = pickerListView.transform.Find ("ScrollableContentPanel").gameObject;
		foreach (Transform child in pickerPanel.transform) {
			GameObject.Destroy(child.gameObject);
		}
		
		float yPosition = -50;
		if (selectedPlayer == null) {
			((RectTransform)pickerPanel.transform).sizeDelta = new Vector2 (0, 100 * (deviceList.Count + 1));

			foreach (KeyValuePair<string, Player> kvp in deviceList) {
				GameObject playerButtonInstance = Instantiate (pickerButton) as GameObject;
				playerButtonInstance.transform.SetParent (pickerPanel.transform);
				playerButtonInstance.transform.localScale = new Vector3 (1, 1, 1);
				((RectTransform)playerButtonInstance.transform).anchoredPosition = new Vector2 (0, yPosition);
				((RectTransform)playerButtonInstance.transform).sizeDelta = new Vector2 (0, 100);
				yPosition = yPosition - 100;
			
				AddPickerListener (playerButtonInstance.GetComponent<Button> (), kvp.Value);
			
				GameObject playerText = playerButtonInstance.transform.Find ("Text").gameObject;
				Text textComponent = playerText.GetComponent<Text> ();
				textComponent.text = kvp.Value.PlayerName;
			}

			GameObject cancelButtonInstance = Instantiate (pickerButton) as GameObject;
			cancelButtonInstance.transform.SetParent (pickerPanel.transform);
			cancelButtonInstance.transform.localScale = new Vector3 (1, 1, 1);
			((RectTransform)cancelButtonInstance.transform).anchoredPosition = new Vector2 (0, yPosition);
			((RectTransform)cancelButtonInstance.transform).sizeDelta = new Vector2 (0, 100);
			yPosition = yPosition - 100;
			
			AddPickerListener (cancelButtonInstance.GetComponent<Button> (), null);
			
			GameObject cancelText = cancelButtonInstance.transform.Find ("Text").gameObject;
			Text cancelTextComponent = cancelText.GetComponent<Text> ();
			cancelTextComponent.text = "Cancel";
		} else {
			((RectTransform)pickerPanel.transform).sizeDelta = new Vector2 (0, 100);
			GameObject disconnectButtonInstance = Instantiate (pickerButton) as GameObject;
			disconnectButtonInstance.transform.SetParent (pickerPanel.transform);
			disconnectButtonInstance.transform.localScale = new Vector3 (1, 1, 1);
			((RectTransform)disconnectButtonInstance.transform).anchoredPosition = new Vector2 (0, yPosition);
			((RectTransform)disconnectButtonInstance.transform).sizeDelta = new Vector2 (0, 100);
			yPosition = yPosition - 100;
			
			AddPickerListener (disconnectButtonInstance.GetComponent<Button> (), null);
			
			GameObject disconnectText = disconnectButtonInstance.transform.Find ("Text").gameObject;
			Text textComponent = disconnectText.GetComponent<Text> ();
			textComponent.text = "Disconnect";
		}
	}
	
	void AddPickerListener(Button button, Player player) {
		button.onClick.AddListener (() => SetDevice (player));
	}

	//Called in a constant loop to draw screen
	//Handle showing the device picker when the user clicks on it
	void OnGUI () {
		Button deviceButton = GameObject.Find ("DeviceButton").GetComponent<Button> ();
		if (selectedPlayer == null) {
			if (deviceList.Count == 0) {
				deviceButton.enabled = false;
				deviceButton.GetComponent<Image>().sprite = null;
			} else {
				deviceButton.enabled = true;
				deviceButton.GetComponent<Image>().sprite = noDeviceSelectedSprite;
			}
		} else {
			deviceButton.GetComponent<Image>().sprite = deviceSelectedSprite;
			deviceButton.enabled = true;
		}

		if (!showDeviceMenu) {
			pickerListView.SetActive (false);
			listView.SetActive (true);
		} else {
			pickerListView.SetActive (true);
			listView.SetActive (false);
		}
	}
	
	private void ActionlessCallback(VoidResult voidResult) {
	}

	public void SetDevice (Player player) {
		if (selectedPlayer != null && player == null) {
			flingService.RemoveStatusListener(selectedPlayer, ActionlessCallback, FlingCallbackError);
			
			Text durationStatus = GameObject.Find ("DurationText").GetComponent<Text> ();
			durationStatus.text = "";

			Slider positionSlider = GameObject.Find ("PositionSlider").GetComponent<Slider> ();
			positionSlider.value = 0;
			
			Text mediaStatus = GameObject.Find ("MediaStatusText").GetComponent<Text> ();
			mediaStatus.text = "";
			
			Text positionStatus = GameObject.Find ("PositionText").GetComponent<Text> ();
			positionStatus.text = "";
		}

		showDeviceMenu = false;
		selectedPlayer = player;
		previousPlayer = null;

		flingService.AddStatusListener (selectedPlayer, ActionlessCallback, FlingCallbackError);

		SetPositionUpdateIntervalParameters setPositionUpdateIntervalParameters = new SetPositionUpdateIntervalParameters ();
		setPositionUpdateIntervalParameters.Player = selectedPlayer;
		setPositionUpdateIntervalParameters.Interval = 1000;
		flingService.SetPositionUpdateInterval (setPositionUpdateIntervalParameters, ActionlessCallback, FlingCallbackError);
	}

	void MediaSelected(Dictionary<string, System.Object> node) {
		sourceParameters = new SetMediaSourceParameters ();
		sourceParameters.Source = node["url"] as string;
		sourceParameters.MetadataJson = Json.Serialize (node["metadata"]);
		sourceParameters.AutoPlay = true;
		sourceParameters.PlayInBg = false;

		if (selectedPlayer != null) {
			sourceParameters.Player = selectedPlayer;
			flingService.SetMediaSource (sourceParameters, ActionlessCallback, FlingCallbackError);
		}
	}

	public void Play () {
		if (selectedPlayer != null && sourceParameters != null) {
			if ( isPaused ) {
				flingService.Play (selectedPlayer, ActionlessCallback, FlingCallbackError);
			} else {
				sourceParameters.Player = selectedPlayer;
				flingService.SetMediaSource (sourceParameters, ActionlessCallback, FlingCallbackError);
			}
		}
	}

	public void Rewind () {
		if (selectedPlayer != null) {
			PlayerSeekParameters seekParameters = new PlayerSeekParameters ();
			seekParameters.Player = selectedPlayer;
			seekParameters.PlayerSeekMode = PlayerSeekMode.PSM_RELATIVE;
			seekParameters.Position = -10000;
			flingService.Seek (seekParameters, ActionlessCallback, FlingCallbackError);
		}
	}

	public void Pause () {
		if (selectedPlayer != null) {
		    flingService.Pause (selectedPlayer, ActionlessCallback, FlingCallbackError);
		}
	}
	
	public void Stop () {
		if (selectedPlayer != null) {
			flingService.Stop (selectedPlayer, ActionlessCallback, FlingCallbackError);
		}
	}
	
	public void FastForward () {
		if (selectedPlayer != null) {
			PlayerSeekParameters seekParameters = new PlayerSeekParameters ();
			seekParameters.Player = selectedPlayer;
			seekParameters.PlayerSeekMode = PlayerSeekMode.PSM_RELATIVE;
			seekParameters.Position = 10000;
			flingService.Seek (seekParameters, ActionlessCallback, FlingCallbackError);
		}
	}
}
